require "deliveryboy/version"

module Deliveryboy
  class DeliveryboyError < ::StandardError; end
  class InvalidChannelError < DeliveryboyError; end
  class MissingHandlerError < DeliveryboyError; end
  class PubSub
    class Subscriber
      def initialize(channel, &handler)
        unless channel.is_a?(String) || channel.is_a?(Regexp)
          raise(InvalidChannelError, "invalid channel")
        end

        unless block_given?
          raise(MissingHandlerError, "expected message handler")
        end

        @channel = channel
        @handler = handler
      end

      def receive(incoming_channel, *message)
        unless incoming_channel.is_a?(String)
          raise(InvalidChannelError, "invalid channel")
        end
        
        if match?(incoming_channel)
          @handler.call(*message)
        end
      end

      private

      def match?(incoming_channel)
        case @channel
        when String
          incoming_channel == @channel
        when Regexp
          incoming_channel.match(@channel)
        end
      end
    end

    def initialize
      @subscribers = []
    end

    def subscribe(*args, &block)
      subscriber = Subscriber.new(*args, &block)
      @subscribers << subscriber
      subscriber
    end

    def publish(*args)
      @subscribers.each do |subscriber| 
        subscriber.receive(*args)
      end
    end

    def remove(subscriber)
      @subscribers.delete(subscriber)
    end
  end
end
